 
require "Window"
require "Apollo"
require "GameLib"
require "Unit"
require "AbilityBook"

-- Remember values:
-- Punish buff spell effect id = 70775
-- Punish ability id = 32336
-- Neutralize buff spell effect id = 58664
-- Decimate buff spell effect id = 70635
-- Ruin buff spell effect id = 38841 
-- Concussive kicks ability id = 23984
-- Concussive kicks buff spell effect id = 53448
 
---------------------
-- Variables
---------------------
local StalkerHelper = {} 
addon_settings = {
	bPunishEnabled = false,
	bNeutralizeEnabled = false,
	bNeutralizeBackground = true,
	vNeutralizeBarColorHex = "ffffffff",
	vNeutralizeTextColorHex = "00000000",
	bRuinEnabled = false,
	bRuinBackground = true,
	vRuinBarColorHex = "ffffffff",
	vRuinTextColorHex = "00000000",
	bDecimateEnabled = false,
	bConcussiveKicksEnabled = false,
	bConcussiveKicksBackground = true,
	vConcussiveKicksBarColorHex = "ffffffff",
	vConcussiveKicksTextColorHex = "00000000",
}
local colors = {}

StalkerHelper.SelfBuffSpellId = {
	Punish = 70775,
	Decimate = 70635,
	Neutralize = 58664,
	ConcussiveKicks = 53448
}

StalkerHelper.SelfDebuffSpellId = {}

StalkerHelper.TargetBuffSpellId = {}

StalkerHelper.TargetDebuffSpellId = {
	-- All ruin debuffs, the number is the represented Tier.
	Ruin  = 38841,
	Ruin1 = 59499,
	Ruin2 = 59500,
	Ruin3 = 59501,
	Ruin4 = 59502,
	Ruin5 = 59503,
	Ruin6 = 59504,
	Ruin7 = 59505,
	Ruin8 = 59506
}


-- Since all trackers work differently, we define a function table with each tracked ability's own function

StalkerHelper.TrackedAbilitiesFunctions = {
	Punish = 
		function(stalkerHelper, activeBuff) 	
			local nResourceCurrent = stalkerHelper.unitPlayer:GetResource(3)
			local cooldown = 8
			
			if stalkerHelper.abilities ~= nil then
			    for key, ability in pairs(stalkerHelper.abilities) do
			        if ability.nId == 32336 and ability.tTiers and ability.tTiers[ability.nCurrentTier] then
			            cooldown = ability.tTiers[ability.nCurrentTier].splObject:GetCooldownRemaining()
			        end
			    end
			end
										
			if nResourceCurrent < 35 and cooldown == 0 then
				stalkerHelper.Indicators["Punish"]:Show(true)
			end
		end,
			
		
				
	Decimate = 
		function(stalkerHelper, activeBuff)
			local cooldown = 5

			if stalkerHelper.abilities ~= nil then
			    for key, ability in pairs(stalkerHelper.abilities) do
			        if ability.nId == 31937 and ability.tTiers and ability.tTiers[ability.nCurrentTier] then
			            cooldown = ability.tTiers[ability.nCurrentTier].splObject:GetCooldownRemaining()
			        end
			    end
			end
		
			if cooldown == 0 then
				stalkerHelper.Indicators["Decimate"]:Show(true)
			end
	   end,
	
	
	
	Neutralize = 
		function(stalkerHelper, activeBuff)
			stalkerHelper.Indicators["Neutralize"]:SetText(activeBuff["nCount"])
			stalkerHelper.Indicators["Neutralize"]:SetProgress(activeBuff["fTimeRemaining"])
			stalkerHelper.Indicators["Neutralize"]:Show(true)
		end,
		
		
		
	ConcussiveKicks = 
		function(stalkerHelper, activeBuff)
			stalkerHelper.Indicators["ConcussiveKicks"]:SetProgress(activeBuff["fTimeRemaining"])
			stalkerHelper.Indicators["ConcussiveKicks"]:Show(true)
		end,
	
	
		
	Ruin = 
		function(stalkerHelper, activeBuff)
			stalkerHelper.Indicators["Ruin"]:SetText(StalkerHelper:Round(activeBuff["fTimeRemaining"], 1))
			stalkerHelper.Indicators["Ruin"]:SetProgress(activeBuff["fTimeRemaining"])
			stalkerHelper.Indicators["Ruin"]:Show(true)
	 	end
}




StalkerHelper.Indicators = {}

local SettingsRef, RuinBarRef, NeutralizeBarRef
 
--------------------
-- Initialization
--------------------
function StalkerHelper:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end


function StalkerHelper:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


function StalkerHelper:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("StalkerHelper.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end


function StalkerHelper:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	
		-- Load all indicators used		
		for indicatorName, value in pairs(self.SelfBuffSpellId) do
			self.Indicators[indicatorName] = Apollo.LoadForm(self.xmlDoc, indicatorName .. "Indicator", nil, self)
			self.Indicators[indicatorName]:Show(false, true)
		end
		
		local previousDebuff = ""
		for indicatorName, value in pairs(self.TargetDebuffSpellId) do
			local currentDebuff = string.gsub(indicatorName, "%d", "")
			if previousDebuff ~= currentDebuff then
				self.Indicators[currentDebuff] = Apollo.LoadForm(self.xmlDoc, currentDebuff .. "Indicator", nil, self)
				self.Indicators[currentDebuff]:Show(false, true)
			else
				previousDebuff = currentDebuff
			end
		end
		
		self.optionsMenu = Apollo.LoadForm(self.xmlDoc, "OptionsMenuWindow", nil, self)
			
		self:OnChangeAbilities()
		
		SettingsRef = addon_settings;
		self.optionsMenu:Show(false, true)
		
		Apollo.RegisterSlashCommand("stalkerhelper", "OnStalkerHelperOn", self)
		Apollo.RegisterSlashCommand("StalkerHelper", "OnStalkerHelperOn", self)
		Apollo.RegisterSlashCommand("sh", "OnStalkerHelperOn", self)
		
		self:OnRequiredFlagsChanged()
		
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)

		self:RestorePlayerPrefs()
	end
end


function StalkerHelper:OnRequiredFlagsChanged()
	if GameLib.GetPlayerUnit() then
		self:OnCharacterCreated()
	else
		Apollo.RegisterEventHandler("CharacterCreated", "OnCharacterCreated", self)
	end
end


function StalkerHelper:OnCharacterCreated()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Stalker then
		return
	end
	
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnFrame", self)
	Apollo.RegisterEventHandler("AbilityBookChange", "OnAbilityBookChangeEvent", self)
	Apollo.RegisterTimerHandler("ChangeAbilities", "OnChangeAbilities", self)
end


function StalkerHelper:OnWindowManagementReady()
	for key, value in pairs(self.Indicators) do
		Event_FireGenericEvent("WindowManagementAdd", {wnd = self.Indicators[key], strName = "StalkerHelper: " .. key .. " indicator"})
	end
end


function StalkerHelper:OnAbilityBookChangeEvent()
	Apollo.CreateTimer("ChangeAbilities", 1, false)
end

function StalkerHelper:OnChangeAbilities()
	self.abilities = AbilityBook.GetAbilitiesList()
	-- TODO, MAKE DYNAMIC PROGRESS BAR DURATION
	self.Indicators["ConcussiveKicks"]:SetMax(8)
	self.Indicators["Neutralize"]:SetMax(8)
	self.Indicators["Ruin"]:SetMax(10)
end

----------------------
-- Save/load data functions
----------------------
 
function StalkerHelper:OnSave(eType) 
  if eType == GameLib.CodeEnumAddonSaveLevel.Character then
    return addon_settings
  end
end
 
function StalkerHelper:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		for key, value in pairs(tSavedData) do
			addon_settings[key] = value
		end
	end
end

function StalkerHelper:RestorePlayerPrefs()	
	--Restore checkboxes for tracking and bar background
	for key, value in pairs(self.Indicators) do
		self.optionsMenu:FindChild(key .. "CheckBox"):SetCheck(addon_settings["b" .. key .. "Enabled"])
		if self.optionsMenu:FindChild(key .. "BackgroundCheckBox") ~= nil then
			self.optionsMenu:FindChild(key .. "BackgroundCheckBox"):SetCheck(addon_settings["b" .. key .. "Background"])
			self.Indicators[key]:SetBarColor(CColor.new(self:ConvertHexToColor(addon_settings["v" .. key .. "BarColorHex"])))
			self.Indicators[key]:SetTextColor(CColor.new(self:ConvertHexToColor(addon_settings["v" .. key .. "TextColorHex"])))
			self:ToggleBarBackground(key)
		end
	end
end

----------------------
-- Main functions
----------------------

function StalkerHelper:OnStalkerHelperOn()
	if self.optionsMenu:IsShown() then
		self:CloseOptionsMenu()
		return
	end
	for key, value in pairs(self.Indicators) do
		self.Indicators[key]:Show(true)
		-- Set progress and names to easier determine color and such
		if type(self.Indicators[key].SetProgress) == "function" then
			self.Indicators[key]:SetProgress(4)
			self.Indicators[key]:SetText(key)
		end
	end
	self.optionsMenu:Show(true)
end

function StalkerHelper:CloseOptionsMenu()
	for key, value in pairs(self.Indicators) do
		self.Indicators[key]:Show(false)
	end
	self.optionsMenu:Show(false)
end

function StalkerHelper:OnFrame()
	if not self.optionsMenu:IsShown() then 
		self.unitPlayer = GameLib.GetPlayerUnit()
		self.buffs = self.unitPlayer:GetBuffs()
		self:CheckSelf()
	end
end


-- Function to check self for buffs/debuffs
function StalkerHelper:CheckSelf()
	for key, value in pairs(self.Indicators) do
		self.Indicators[key]:Show(false)
	end
	
	if self.abilities ~= nil then
	    -- Check self for active buffs
		for _, activeBuff in pairs(self.buffs.arBeneficial) do
			for buffName, buffId in pairs(self.SelfBuffSpellId) do
				if activeBuff.splEffect:GetId() == buffId and addon_settings["b" .. buffName.. "Enabled"] then
					-- The buff is a buff we want to track, so called it's function
					self.TrackedAbilitiesFunctions[buffName](self, activeBuff)
	        	end
			end
		end
	else
		Print("AbilityBook is null")
		return
	end

	
	-- Check enemy for active debuffs
	local playerTarget = self.unitPlayer:GetTarget()
	
	-- If no target, hide the bar and just return
	if playerTarget == nil then
		--self.ruinBar:Show(false)
		return
	end
	
	local enemyDebuffs = playerTarget:GetBuffs()
	
	for index, debuff in pairs(enemyDebuffs.arHarmful) do
		for debuffName, debuffId in pairs(self.TargetDebuffSpellId) do
			if debuff.splEffect:GetId() == debuffId and addon_settings["b" .. string.gsub(debuffName, "%d", "") .. "Enabled"] then
				self.TrackedAbilitiesFunctions[string.gsub(debuffName, "%d", "")](self, debuff)
			end
		end
	end
end





----------------------
-- Helper functions
----------------------

function StalkerHelper:Round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end


function StalkerHelper:ConvertColorToHex(color)
	return string.format("%02x%02x%02x%02x", self:Round(255 * color.a), self:Round(255 * color.r), self:Round(255 * color.g), self:Round(255 * color.b))
end


function StalkerHelper:ConvertHexToColor(hex)
	local a = string.sub(hex, 1, 2)
	hex = string.sub(hex, -6)
	local r, g, b = 0, 0, 0
	local n = tonumber(hex, 16)
	if n then
		r = math.floor(n / 65536)
		g = math.floor((n / 256) % 256)
		b = n % 256
	end
	return r / 255, g / 255, b / 255, 1
end


function StalkerHelper:HasColorPicker()
	if ColorPicker then
		return true
	end
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, "You can only change colors if you have ColorPicker installed")
	return false
end



----------------------
-- Options menu functions
----------------------

function StalkerHelper:OnToggleIndicator( wndHandler, wndControl, eMouseButton )
	local indicatorName = string.gsub(wndControl:GetName(), "CheckBox", "")
	addon_settings["b" .. indicatorName .. "Enabled"] = wndControl:IsChecked()
end


function StalkerHelper:OnToggleBarBackground( wndHandler, wndControl, eMouseButtion )
	local barName = string.gsub(wndControl:GetName(), "BackgroundCheckBox", "")
	addon_settings["b" .. barName .. "Background"] = wndControl:IsChecked()
	self:ToggleBarBackground(barName)
end

function StalkerHelper:ToggleBarBackground(barName)
	if addon_settings["b" .. barName .. "Background"] then
		self.Indicators[barName]:SetSprite("CRB_UIKitSprites:spr_iconslotEMPTY")
	else
		self.Indicators[barName]:SetSprite("")
	end
end


function StalkerHelper:OnCloseOptionsMenu( wndHandler, wndControl, eMouseButton )
	self:CloseOptionsMenu()
end


function StalkerHelper:OnChangeBarColor( wndHandler, wndControl, eMouseButton )
	if self:HasColorPicker() then
		abilityName = string.gsub(wndControl:GetName(), "ColorBtn", "")
		colors.tmpColor = CColor.new(StalkerHelper:ConvertHexToColor(addon_settings["v" .. abilityName .. "BarColorHex"]))
		ColorPicker.AdjustCColor(colors.tmpColor , false, UpdateBarColor)
	end
end

function UpdateBarColor()
	addon_settings["v" .. abilityName .. "BarColorHex"] = StalkerHelper:ConvertColorToHex(colors.tmpColor)
	StalkerHelper.Indicators[abilityName]:SetBarColor(colors.tmpColor)
end


function StalkerHelper:OnChangeBarTextColor( wndHandler, wndControl, eMouseButton )
	if self:HasColorPicker() then
		abilityName = string.gsub(wndControl:GetName(), "TextColorBtn", "")
		colors.tmpColor = CColor.new(StalkerHelper:ConvertHexToColor(addon_settings["v" .. abilityName .. "TextColorHex"]))
		ColorPicker.AdjustCColor(colors.tmpColor , false, UpdateBarTextColor)
	end
end

function UpdateBarTextColor()
	addon_settings["v" .. abilityName .. "TextColorHex"] = StalkerHelper:ConvertColorToHex(colors.tmpColor)
	StalkerHelper.Indicators[abilityName]:SetTextColor(colors.tmpColor)
end


----------------------
-- StalkerHelper Instance
----------------------
local StalkerHelperInst = StalkerHelper:new()
StalkerHelperInst:Init()

